package net.sourceforge.cathcart.taskdefs;

import java.io.File;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.List;
import java.util.Map;

import javax.xml.parsers.DocumentBuilderFactory;

import net.sourceforge.cathcart.TaskParser;
import net.sourceforge.cathcart.typedefs.DownwardComparison;
import net.sourceforge.cathcart.typedefs.UpwardComparison;
import net.sourceforge.cathcart.typedefs.PropertyType;
import net.sourceforge.cathcart.typedefs.SizeComparison;
import net.sourceforge.cathcart.typedefs.Trend;

import org.apache.tools.ant.BuildException;
import org.apache.tools.ant.Project;
import org.apache.tools.ant.Task;
import org.apache.tools.ant.taskdefs.Copy;
import org.apache.tools.ant.taskdefs.Mkdir;
import org.apache.tools.ant.taskdefs.Move;
import org.apache.tools.ant.taskdefs.XSLTProcess;
import org.apache.tools.ant.taskdefs.XmlProperty;
import org.apache.tools.ant.types.Path;
import org.apache.tools.ant.types.Resource;
import org.apache.tools.ant.types.ResourceCollection;
import org.apache.tools.ant.types.XMLCatalog;
import org.apache.tools.ant.types.resources.JavaResource;

/**
 * Base class for all Cathcart tasks
 * 
 * @author Steven Hale
 * 
 */
public abstract class CheckBase extends Task {
	protected String tool;
	protected String classpath;
	protected String storeDir = "previous";
	protected String inXml;
	protected String outXml;
	protected String defaultName;
	protected boolean backup = true;
	protected boolean haltOnFailure = true;
	protected Log logger = new Log(this);

	public static final String TOOLS_CONFIG = "/tools_config.xml";

	protected Map<String, PropertyType> allowedPropertyTypes;
	protected XMLCatalog xmlCatalog = new XMLCatalog();
	private final SimpleDateFormat sdf = new SimpleDateFormat(
			"_yyyy-MM-dd_HH.mm.ss.S");

	List<PropertyType> propertyTypes = new ArrayList<PropertyType>();

	public String getDefaultName() {
		if (defaultName == null) {
			defaultName = "cathcart_" + tool;
		}
		return defaultName;
	}

	void setClasspath() {
		if (classpath == null) {
			classpath = getProject().getProperty("ant.home")
					+ "/lib/cathcart.jar";
		}
		logger.log("Classpath for xsl " + classpath, Project.MSG_VERBOSE);
	}

	void setComparisonArguments(PropertyType propertyType, String property,
			Double limit, SizeComparison sizeComparison) {
		sizeComparison.setOperandOne(new Double(property));
		sizeComparison.setOperandTwo(limit);
		sizeComparison.setLeeway(propertyType.getLeeway());
		sizeComparison.setAbsolute(propertyType.isAbsolute());
	}

	void checkNestedElement(PropertyType propertyType, boolean allowLimit) {
		if ((!allowLimit) && (propertyType.getLimit() != 0)) {
			throw new BuildException(
					"limit attribute cannot be used when comparing to previous values");
		}
	}

	boolean allowedPropertyType(PropertyType propertyType) {
		if (propertyType.isAlwaysSuccessful()) {
			return true;
		}
		if (allowedPropertyTypes.containsKey(propertyType.getName())) {
			return true;
		}
		return false;

	}

	void performCopy(Copy task, File out) {
		init(task);
		task.setFile(out);
		task.setTodir(new File(storeDir));
		task.perform();
	}

	void performMove(Move task, File out) {
		init(task);
		task.setFile(new File(storeDir, out.getName()));
		Calendar cal = Calendar.getInstance();
		task.setTofile(new File(storeDir, out.getName()
				+ sdf.format(cal.getTime())));
		task.perform();

	}

	void performXmlProperty(XmlProperty task, File src, String prefix) {
		init(task);
		task.addConfiguredXMLCatalog(xmlCatalog);
		task.setFile(src);
		task.setPrefix(prefix);
		task.perform();
	}

	void performXslt(XSLTProcess task, File outFile) {
		File inFile = new File(inXml);
		init(task);
		task.addConfiguredXMLCatalog(xmlCatalog);
		task.setIn(inFile);
		task.setOut(outFile);
		task.setForce(true);

		ResourceCollection resourceCollection = new JavaResource(
				getDefaultName() + ".xsl", new Path(getProject(), classpath));
		Path path = new Path(getProject(), classpath);
		Resource resource = new JavaResource(getDefaultName() + ".xsl", path);
		resource.setProject(getProject());

		task.setXslResource(resource);
		task.add(resourceCollection);
		task.perform();

	}

	void performMkdir(Mkdir task) {
		init(task);
		task.setDir(new File(storeDir));
		task.perform();
	}

	void init(Task task) {
		task.bindToOwner(this);
		task.init();
	}

	public void setClasspath(String classpath) {
		this.classpath = classpath;
	}

	public void setStoreDir(String storeDir) {
		this.storeDir = storeDir;
	}

	public void setInXml(String inXml) {
		this.inXml = inXml;
	}

	public void setOutXml(String outXml) {
		this.outXml = outXml;
	}

	public String getOutXml() {
		return outXml;
	}

	public void addConfigured(PropertyType propertyType) {
		logger.log("Adding " + propertyType.getName(), Project.MSG_VERBOSE);
		propertyTypes.add(propertyType);

	}

	SizeComparison chooseComparison(PropertyType propertyType) {
		SizeComparison sizeComparison = new DownwardComparison();
		if (Trend.valueOf(propertyType.getDesiredTrend()).equals(Trend.upward)) {
			logger.log("Using UpwardComparison", Project.MSG_VERBOSE);
			sizeComparison = new UpwardComparison();
		}
		return sizeComparison;
	}

	/**
	 * add an XMLCatalog as a nested element; optional.
	 * 
	 * @param catalog
	 *            the XMLCatalog to use
	 */
	public void addConfiguredXMLCatalog(XMLCatalog catalog) {
		xmlCatalog.addConfiguredXMLCatalog(catalog);
	}

	public void setAllowedPropertyTypes(
			Map<String, PropertyType> allowedPropertyTypes) {
		this.allowedPropertyTypes = allowedPropertyTypes;
	}

	public Map<String, PropertyType> getAllowedPropertyTypes() {
		return allowedPropertyTypes;
	}

	public void setBackup(boolean backup) {
		this.backup = backup;
	}

	public void setHaltOnFailure(boolean haltOnFailure) {
		this.haltOnFailure = haltOnFailure;
	}

	protected void failure(PropertyType propertyType,
			SizeComparison sizeComparison) {
		String message = propertyType.getName()
				+ ": Comparison to previous result FAILED: "
				+ sizeComparison.failedMessage();
		if (haltOnFailure) {
			throw new BuildException(message);
		} else {
			logger.log(message, Project.MSG_ERR);
		}
	}

	protected void success(PropertyType propertyType,
			SizeComparison sizeComparison) {
		String message = propertyType.getName() + ": Comparison to limit OK: "
				+ sizeComparison.toString();
		logger.log(message, Project.MSG_VERBOSE);
	}

	protected void success() {
		logger.log("always successful comparison OK", Project.MSG_VERBOSE);
	}

	public void setTool(String tool) {
		this.tool = tool;
	}

	public void parseConfig() {

		TaskParser parser = new TaskParser(CheckBase.class
				.getResourceAsStream(TOOLS_CONFIG), DocumentBuilderFactory
				.newInstance());
		setAllowedPropertyTypes(parser.getPropertyTypes(parser.parse(), tool));
	}

	public void fixPropertyTypes() {
		for (PropertyType property : propertyTypes) {
			logger.log("fixing " + property.getName(), Project.MSG_VERBOSE);
			PropertyType propertyType = allowedPropertyTypes.get(property
					.getName());
			if (propertyType == null) {
				String message = "property " + property.getName()
						+ " not defined";
				throw new BuildException(message);

			}

			if (property.getDesiredTrend() == null) {
				logger.log("setting trend " + property.getName() + " "
						+ propertyType.getDesiredTrend(), Project.MSG_VERBOSE);

				property.setDesiredTrend(propertyType.getDesiredTrend());
			}

		}
	}

	public void setLog(Log log) {
		this.logger = log;
	}

}
